@@ -330,7 +330,7 @@ static ssize_t PrintChannelLocations(FILE *file,const Image *image,
       if (traits == UndefinedPixelTrait)
         continue;
       offset=GetPixelChannelOffset(image,channel);
-      match=fabs((double) (p[offset]-target)) < 0.5 ? MagickTrue : MagickFalse;
+      match=fabs((double) p[offset]-target) < 0.5 ? MagickTrue : MagickFalse;
       if (match != MagickFalse)
         {
           if ((max_locations != 0) && (n >= (ssize_t) max_locations))
@@ -567,31 +567,39 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         case RGBColorspace:
         case sRGBColorspace:
         {
-          (void) PrintChannelLocations(file,image,RedPixelChannel,"Red",
-            statistic_type,max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,GreenPixelChannel,"Green",
-            statistic_type,max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,BluePixelChannel,"Blue",
-            statistic_type,max_locations,channel_statistics);
+          if ((GetPixelRedTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelLocations(file,image,RedPixelChannel,"Red",
+              statistic_type,max_locations,channel_statistics);
+          if ((GetPixelGreenTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelLocations(file,image,GreenPixelChannel,"Green",
+              statistic_type,max_locations,channel_statistics);
+          if ((GetPixelBlueTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelLocations(file,image,BluePixelChannel,"Blue",
+              statistic_type,max_locations,channel_statistics);
           break;
         }
         case CMYKColorspace:
         {
-          (void) PrintChannelLocations(file,image,CyanPixelChannel,"Cyan",
-            statistic_type,max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,MagentaPixelChannel,
-            "Magenta",statistic_type,max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,YellowPixelChannel,"Yellow",
-            statistic_type,max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,BlackPixelChannel,"Black",
-            statistic_type,max_locations,channel_statistics);
+          if ((GetPixelCyanTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelLocations(file,image,CyanPixelChannel,"Cyan",
+              statistic_type,max_locations,channel_statistics);
+          if ((GetPixelMagentaTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelLocations(file,image,MagentaPixelChannel,
+              "Magenta",statistic_type,max_locations,channel_statistics);
+          if ((GetPixelYellowTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelLocations(file,image,YellowPixelChannel,"Yellow",
+              statistic_type,max_locations,channel_statistics);
+          if ((GetPixelBlackTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelLocations(file,image,BlackPixelChannel,"Black",
+              statistic_type,max_locations,channel_statistics);
           break;
         }
         case LinearGRAYColorspace:
         case GRAYColorspace:
         {
-          (void) PrintChannelLocations(file,image,GrayPixelChannel,"Gray",
-            statistic_type,max_locations,channel_statistics);
+          if ((GetPixelGrayTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelLocations(file,image,GrayPixelChannel,"Gray",
+              statistic_type,max_locations,channel_statistics);
           break;
         }
         default:
@@ -599,10 +607,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
           {
             PixelChannel channel = GetPixelChannelChannel(image,i);
-            if (channel == AlphaPixelChannel)
-              continue;
-            (void) PrintChannelLocations(file,image,channel,"Channel",
-              statistic_type,max_locations,channel_statistics);
+            PixelTrait traits = GetPixelChannelTraits(image,channel);
+            if ((traits & UpdatePixelTrait) != 0)
+              (void) PrintChannelLocations(file,image,channel,"Channel",
+                statistic_type,max_locations,channel_statistics);
           }
           break;
         }
@@ -798,31 +806,39 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         case RGBColorspace:
         case sRGBColorspace:
         {
-          (void) FormatLocaleFile(file,"    Red: %.20g-bit\n",(double)
-            channel_statistics[RedPixelChannel].depth);
-          (void) FormatLocaleFile(file,"    Green: %.20g-bit\n",(double)
-            channel_statistics[GreenPixelChannel].depth);
-          (void) FormatLocaleFile(file,"    Blue: %.20g-bit\n",(double)
-            channel_statistics[BluePixelChannel].depth);
+          if ((GetPixelRedTraits(image) & UpdatePixelTrait) != 0)
+            (void) FormatLocaleFile(file,"    Red: %.20g-bit\n",(double)
+              channel_statistics[RedPixelChannel].depth);
+          if ((GetPixelGreenTraits(image) & UpdatePixelTrait) != 0)
+            (void) FormatLocaleFile(file,"    Green: %.20g-bit\n",(double)
+              channel_statistics[GreenPixelChannel].depth);
+          if ((GetPixelBlueTraits(image) & UpdatePixelTrait) != 0)
+            (void) FormatLocaleFile(file,"    Blue: %.20g-bit\n",(double)
+              channel_statistics[BluePixelChannel].depth);
           break;
         }
         case CMYKColorspace:
         {
-          (void) FormatLocaleFile(file,"    Cyan: %.20g-bit\n",(double)
-            channel_statistics[CyanPixelChannel].depth);
-          (void) FormatLocaleFile(file,"    Magenta: %.20g-bit\n",(double)
-            channel_statistics[MagentaPixelChannel].depth);
-          (void) FormatLocaleFile(file,"    Yellow: %.20g-bit\n",(double)
-            channel_statistics[YellowPixelChannel].depth);
-          (void) FormatLocaleFile(file,"    Black: %.20g-bit\n",(double)
-            channel_statistics[BlackPixelChannel].depth);
+          if ((GetPixelCyanTraits(image) & UpdatePixelTrait) != 0)
+            (void) FormatLocaleFile(file,"    Cyan: %.20g-bit\n",(double)
+              channel_statistics[CyanPixelChannel].depth);
+          if ((GetPixelMagentaTraits(image) & UpdatePixelTrait) != 0)
+            (void) FormatLocaleFile(file,"    Magenta: %.20g-bit\n",(double)
+              channel_statistics[MagentaPixelChannel].depth);
+          if ((GetPixelYellowTraits(image) & UpdatePixelTrait) != 0)
+            (void) FormatLocaleFile(file,"    Yellow: %.20g-bit\n",(double)
+              channel_statistics[YellowPixelChannel].depth);
+          if ((GetPixelBlackTraits(image) & UpdatePixelTrait) != 0)
+            (void) FormatLocaleFile(file,"    Black: %.20g-bit\n",(double)
+              channel_statistics[BlackPixelChannel].depth);
           break;
         }
         case LinearGRAYColorspace:
         case GRAYColorspace:
         {
-          (void) FormatLocaleFile(file,"    Gray: %.20g-bit\n",(double)
-            channel_statistics[GrayPixelChannel].depth);
+          if ((GetPixelGrayTraits(image) & UpdatePixelTrait) != 0)
+            (void) FormatLocaleFile(file,"    Gray: %.20g-bit\n",(double)
+              channel_statistics[GrayPixelChannel].depth);
           break;
         }
         default:
@@ -830,13 +846,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
           {
             PixelChannel channel = GetPixelChannelChannel(image,i);
-            if ((channel == AlphaPixelChannel) ||
-                (channel == ReadMaskPixelChannel) ||
-                (channel == WriteMaskPixelChannel) ||
-                (channel == CompositeMaskPixelChannel))
-              continue;
-            (void) FormatLocaleFile(file,"    Channel %.20g: %.20g-bit\n",
-              (double) i,(double) channel_statistics[channel].depth);
+            PixelTrait traits = GetPixelChannelTraits(image,channel);
+            if ((traits & UpdatePixelTrait) != 0)
+              (void) FormatLocaleFile(file,"    Channel %.20g: %.20g-bit\n",
+                (double) i,(double) channel_statistics[channel].depth);
           }
           break;
         }
@@ -874,31 +887,39 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         case RGBColorspace:
         case sRGBColorspace:
         {
-          (void) PrintChannelStatistics(file,RedPixelChannel,"Red",1.0/
-            scale,channel_statistics);
-          (void) PrintChannelStatistics(file,GreenPixelChannel,"Green",1.0/
-            scale,channel_statistics);
-          (void) PrintChannelStatistics(file,BluePixelChannel,"Blue",1.0/
-            scale,channel_statistics);
+          if ((GetPixelRedTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelStatistics(file,RedPixelChannel,"Red",1.0/
+              scale,channel_statistics);
+          if ((GetPixelGreenTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelStatistics(file,GreenPixelChannel,"Green",1.0/
+              scale,channel_statistics);
+          if ((GetPixelBlueTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelStatistics(file,BluePixelChannel,"Blue",1.0/
+              scale,channel_statistics);
           break;
         }
         case CMYKColorspace:
         {
-          (void) PrintChannelStatistics(file,CyanPixelChannel,"Cyan",1.0/
-            scale,channel_statistics);
-          (void) PrintChannelStatistics(file,MagentaPixelChannel,"Magenta",1.0/
-            scale,channel_statistics);
-          (void) PrintChannelStatistics(file,YellowPixelChannel,"Yellow",1.0/
-            scale,channel_statistics);
-          (void) PrintChannelStatistics(file,BlackPixelChannel,"Black",1.0/
-            scale,channel_statistics);
+          if ((GetPixelCyanTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelStatistics(file,CyanPixelChannel,"Cyan",1.0/
+              scale,channel_statistics);
+          if ((GetPixelMagentaTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelStatistics(file,MagentaPixelChannel,"Magenta",
+              1.0/scale,channel_statistics);
+          if ((GetPixelYellowTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelStatistics(file,YellowPixelChannel,"Yellow",1.0/
+              scale,channel_statistics);
+          if ((GetPixelBlackTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelStatistics(file,BlackPixelChannel,"Black",1.0/
+              scale,channel_statistics);
           break;
         }
         case LinearGRAYColorspace:
         case GRAYColorspace:
         {
-          (void) PrintChannelStatistics(file,GrayPixelChannel,"Gray",1.0/
-            scale,channel_statistics);
+          if ((GetPixelGrayTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelStatistics(file,GrayPixelChannel,"Gray",1.0/
+              scale,channel_statistics);
           break;
         }
         default:
@@ -909,15 +930,14 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
               label[MagickPathExtent];
 
             PixelChannel channel = GetPixelChannelChannel(image,i);
-            if ((channel == AlphaPixelChannel) ||
-                (channel == ReadMaskPixelChannel) ||
-                (channel == WriteMaskPixelChannel) ||
-                (channel == CompositeMaskPixelChannel))
-              continue;
-            (void) FormatLocaleString(label,MagickPathExtent,"Channel %.20g",
-              (double) i);
-            (void) PrintChannelStatistics(file,channel,label,1.0/scale,
-              channel_statistics);
+            PixelTrait traits = GetPixelChannelTraits(image,channel);
+            if ((traits & UpdatePixelTrait) != 0)
+              {
+                (void) FormatLocaleString(label,MagickPathExtent,
+                  "Channel %.20g",(double) i);
+                (void) PrintChannelStatistics(file,channel,label,1.0/scale,
+                  channel_statistics);
+              }
           }
           break;
         }
@@ -967,30 +987,38 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         case RGBColorspace:
         case sRGBColorspace:
         {
-          (void) PrintChannelMoments(file,RedPixelChannel,"Red",scale,
-            channel_moments);
-          (void) PrintChannelMoments(file,GreenPixelChannel,"Green",scale,
-            channel_moments);
-          (void) PrintChannelMoments(file,BluePixelChannel,"Blue",scale,
-            channel_moments);
+          if ((GetPixelRedTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelMoments(file,RedPixelChannel,"Red",scale,
+              channel_moments);
+          if ((GetPixelGreenTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelMoments(file,GreenPixelChannel,"Green",scale,
+              channel_moments);
+          if ((GetPixelBlueTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelMoments(file,BluePixelChannel,"Blue",scale,
+              channel_moments);
           break;
         }
         case CMYKColorspace:
         {
-          (void) PrintChannelMoments(file,CyanPixelChannel,"Cyan",scale,
-            channel_moments);
-          (void) PrintChannelMoments(file,MagentaPixelChannel,"Magenta",scale,
-            channel_moments);
-          (void) PrintChannelMoments(file,YellowPixelChannel,"Yellow",scale,
-            channel_moments);
-          (void) PrintChannelMoments(file,BlackPixelChannel,"Black",scale,
-            channel_moments);
+          if ((GetPixelCyanTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelMoments(file,CyanPixelChannel,"Cyan",scale,
+              channel_moments);
+          if ((GetPixelMagentaTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelMoments(file,MagentaPixelChannel,"Magenta",scale,
+              channel_moments);
+          if ((GetPixelYellowTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelMoments(file,YellowPixelChannel,"Yellow",scale,
+              channel_moments);
+          if ((GetPixelBlackTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelMoments(file,BlackPixelChannel,"Black",scale,
+              channel_moments);
           break;
         }
         case GRAYColorspace:
         {
-          (void) PrintChannelMoments(file,GrayPixelChannel,"Gray",scale,
-            channel_moments);
+          if ((GetPixelGrayTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelMoments(file,GrayPixelChannel,"Gray",scale,
+              channel_moments);
           break;
         }
         default:
@@ -1001,12 +1029,14 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
               label[MagickPathExtent];
 
             PixelChannel channel = GetPixelChannelChannel(image,i);
-            if (channel == AlphaPixelChannel)
-              continue;
-            (void) FormatLocaleString(label,MagickPathExtent,"Channel %.20g",
-              (double) i);
-            (void) PrintChannelMoments(file,channel,label,scale,
-              channel_moments);
+            PixelTrait traits = GetPixelChannelTraits(image,channel);
+            if ((traits & UpdatePixelTrait) != 0)
+              {
+                (void) FormatLocaleString(label,MagickPathExtent,
+                  "Channel %.20g",(double) i);
+                (void) PrintChannelMoments(file,channel,label,scale,
+                  channel_moments);
+              }
           }
           break;
         }
@@ -1038,30 +1068,38 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         case RGBColorspace:
         case sRGBColorspace:
         {
-          (void) PrintChannelFeatures(file,RedPixelChannel,"Red",
-            channel_features);
-          (void) PrintChannelFeatures(file,GreenPixelChannel,"Green",
-            channel_features);
-          (void) PrintChannelFeatures(file,BluePixelChannel,"Blue",
-            channel_features);
+          if ((GetPixelRedTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelFeatures(file,RedPixelChannel,"Red",
+              channel_features);
+          if ((GetPixelGreenTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelFeatures(file,GreenPixelChannel,"Green",
+              channel_features);
+          if ((GetPixelBlueTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelFeatures(file,BluePixelChannel,"Blue",
+              channel_features);
           break;
         }
         case CMYKColorspace:
         {
-          (void) PrintChannelFeatures(file,CyanPixelChannel,"Cyan",
-            channel_features);
-          (void) PrintChannelFeatures(file,MagentaPixelChannel,"Magenta",
-            channel_features);
-          (void) PrintChannelFeatures(file,YellowPixelChannel,"Yellow",
-            channel_features);
-          (void) PrintChannelFeatures(file,BlackPixelChannel,"Black",
-            channel_features);
+          if ((GetPixelCyanTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelFeatures(file,CyanPixelChannel,"Cyan",
+              channel_features);
+          if ((GetPixelMagentaTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelFeatures(file,MagentaPixelChannel,"Magenta",
+              channel_features);
+          if ((GetPixelYellowTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelFeatures(file,YellowPixelChannel,"Yellow",
+              channel_features);
+          if ((GetPixelBlackTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelFeatures(file,BlackPixelChannel,"Black",
+              channel_features);
           break;
         }
         case GRAYColorspace:
         {
-          (void) PrintChannelFeatures(file,GrayPixelChannel,"Gray",
-            channel_features);
+          if ((GetPixelGrayTraits(image) & UpdatePixelTrait) != 0)
+            (void) PrintChannelFeatures(file,GrayPixelChannel,"Gray",
+              channel_features);
           break;
         }
         default:
@@ -1072,11 +1110,14 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
               label[MagickPathExtent];
 
             PixelChannel channel = GetPixelChannelChannel(image,i);
-            if (channel == AlphaPixelChannel)
-              continue;
-            (void) FormatLocaleString(label,MagickPathExtent,"Channel %.20g",
-              (double) i);
-            (void) PrintChannelFeatures(file,channel,label,channel_features);
+            PixelTrait traits = GetPixelChannelTraits(image,channel);
+            if ((traits & UpdatePixelTrait) != 0)
+              {
+                (void) FormatLocaleString(label,MagickPathExtent,
+                  "Channel %.20g",(double) i);
+                (void) PrintChannelFeatures(file,channel,label,
+                  channel_features);
+              }
           }
           break;
         }
@@ -1225,15 +1266,18 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         Display image chromaticity.
       */
       (void) FormatLocaleFile(file,"  Chromaticity:\n");
-      (void) FormatLocaleFile(file,"    red primary: (%g,%g)\n",
-        image->chromaticity.red_primary.x,image->chromaticity.red_primary.y);
-      (void) FormatLocaleFile(file,"    green primary: (%g,%g)\n",
-        image->chromaticity.green_primary.x,
-        image->chromaticity.green_primary.y);
-      (void) FormatLocaleFile(file,"    blue primary: (%g,%g)\n",
-        image->chromaticity.blue_primary.x,image->chromaticity.blue_primary.y);
-      (void) FormatLocaleFile(file,"    white point: (%g,%g)\n",
-        image->chromaticity.white_point.x,image->chromaticity.white_point.y);
+      (void) FormatLocaleFile(file,"    red primary: (%g,%g,%g)\n",
+        image->chromaticity.red_primary.x,image->chromaticity.red_primary.y,
+        image->chromaticity.red_primary.z);
+      (void) FormatLocaleFile(file,"    green primary: (%g,%g,%g)\n",
+        image->chromaticity.green_primary.x,image->chromaticity.green_primary.y,
+        image->chromaticity.green_primary.z);
+      (void) FormatLocaleFile(file,"    blue primary: (%g,%g,%g)\n",
+        image->chromaticity.blue_primary.x,image->chromaticity.blue_primary.y,
+        image->chromaticity.blue_primary.z);
+      (void) FormatLocaleFile(file,"    white point: (%g,%g,%g)\n",
+        image->chromaticity.white_point.x,image->chromaticity.white_point.y,
+        image->chromaticity.white_point.z);
     }
   if ((image->extract_info.width*image->extract_info.height) != 0)
     (void) FormatLocaleFile(file,"  Tile geometry: %.20gx%.20g%+.20g%+.20g\n",
@@ -1516,7 +1560,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
                 (double) dataset,(double) record);
               length=(size_t) (GetStringInfoDatum(profile)[i++] << 8);
               length|=GetStringInfoDatum(profile)[i++];
-              length=MagickMin(length,profile_length-i);
+              length=MagickMin(length,profile_length-(size_t) i);
               attribute=(char *) NULL;
               if (~length >= (MagickPathExtent-1))
                 attribute=(char *) AcquireQuantumMemory(length+MagickPathExtent,
