@@ -73,6 +73,7 @@
 #define MagickPathTemplate "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"  /* min 6 X's */
 #define NumberOfResourceTypes  \
   (sizeof(resource_semaphore)/sizeof(*resource_semaphore))
+#define TimeToLiveMax  INT_MAX
 
 /*
   Typedef declarations.
@@ -136,7 +137,7 @@ static ResourceInfo
     MagickULLConstant(768),            /* file limit */
     MagickULLConstant(1),              /* thread limit */
     MagickULLConstant(0),              /* throttle limit */
-    (MagickSizeType) MAGICK_SSIZE_MAX                   /* time limit */
+    TimeToLiveMax                      /* time limit */
   };
 
 static SemaphoreInfo
@@ -332,7 +333,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       if (((MagickSizeType) resource_info.time+(MagickSizeType) request) > (MagickSizeType) resource_info.time)
         {
           resource_info.time+=request;
-          if ((limit == MagickResourceInfinity) ||
+          if ((limit == TimeToLiveMax) ||
               (resource_info.time < (MagickOffsetType) limit))
             status=MagickTrue;
           else
@@ -873,6 +874,65 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+static void FormatTimeToLive(const MagickSizeType ttl,char *timeString)
+{
+  MagickSizeType
+    days,
+    hours,
+    minutes,
+    months,
+    seconds,
+    weeks,
+    years;
+
+  years=ttl/31536000;
+  seconds=ttl % 31536000;
+  if (seconds == 0)
+    {
+      (void) FormatLocaleString(timeString,MagickPathExtent,"%lld years",years);
+      return;
+    }
+  months=ttl/2628000;
+  seconds=ttl % 2628000;
+  if (seconds == 0)
+    {
+      (void) FormatLocaleString(timeString,MagickPathExtent,"%lld months",
+        months);
+      return;
+    }
+  weeks=ttl/604800;
+  seconds=ttl % 604800;
+  if (seconds == 0)
+    {
+      (void) FormatLocaleString(timeString,MagickPathExtent,"%lld weeks",weeks);
+      return;
+    }
+  days=ttl/86400;
+  seconds=ttl % 86400;
+  if (seconds == 0)
+    {
+      (void) FormatLocaleString(timeString,MagickPathExtent,"%lld days",days);
+      return;
+    }
+  hours=ttl/3600;
+  seconds=ttl % 3600;
+  if (seconds == 0)
+    {
+      (void) FormatLocaleString(timeString,MagickPathExtent,"%lld hours",hours);
+      return;
+    }
+  minutes=ttl/60;
+  seconds=ttl % 60;
+  if (seconds == 0)
+    {
+      (void) FormatLocaleString(timeString,MagickPathExtent,"%lld minutes",
+        minutes);
+      return;
+    }
+  (void) FormatLocaleString(timeString,MagickPathExtent,"%lld seconds",ttl);
+}
+
 MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
   ExceptionInfo *magick_unused(exception))
 {
@@ -912,9 +972,8 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
     (void) FormatMagickSize(resource_info.disk_limit,MagickTrue,"B",
       MagickFormatExtent,disk_limit);
   (void) CopyMagickString(time_limit,"unlimited",MagickFormatExtent);
-  if (resource_info.time_limit != MagickResourceInfinity)
-    (void) FormatLocaleString(time_limit,MagickFormatExtent,"%.20g",(double)
-      ((MagickOffsetType) resource_info.time_limit));
+  if (resource_info.time_limit != TimeToLiveMax)
+    FormatTimeToLive(resource_info.time_limit,time_limit);
   (void) FormatLocaleFile(file,"Resource limits:\n");
   (void) FormatLocaleFile(file,"  Width: %s\n",width_limit);
   (void) FormatLocaleFile(file,"  Height: %s\n",height_limit);
@@ -1275,12 +1334,12 @@ MagickPrivate MagickBooleanType ResourceComponentGenesis(void)
         limit,100.0));
       limit=DestroyString(limit);
     }
-  (void) SetMagickResourceLimit(TimeResource,MagickResourceInfinity);
+  (void) SetMagickResourceLimit(TimeResource,TimeToLiveMax);
   limit=GetEnvironmentValue("MAGICK_TIME_LIMIT");
   if (limit != (char *) NULL)
     {
-      (void) SetMagickResourceLimit(TimeResource,StringToMagickSizeType(limit,
-        100.0));
+      (void) SetMagickResourceLimit(TimeResource,(MagickSizeType)
+        ParseMagickTimeToLive(limit));
       limit=DestroyString(limit);
     }
   (void) SetMagickResourceLimit(ListLengthResource,MagickResourceInfinity);
@@ -1486,10 +1545,8 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
       if (value == (char *) NULL)
         resource_info.time_limit=limit;
       else
-        resource_info.time_limit=MagickMin(limit,StringToMagickSizeType(value,
-          100.0));
-      resource_info.time_limit=MagickMin(resource_info.time_limit,
-        (MagickSizeType) MAGICK_SSIZE_MAX);
+        resource_info.time_limit=MagickMin(limit,(MagickSizeType)
+          ParseMagickTimeToLive(value));
       break;
     }
     case WidthResource:
