@@ -350,7 +350,7 @@ MagickExport double GetElapsedTime(TimerInfo *time_info)
 MagickExport time_t GetMagickTime(void)
 {
   static time_t
-    constant_magick_time = (time_t) 0;
+    magick_epoch = (time_t) 0;
 
   static MagickBooleanType
     epoch_initialized = MagickFalse;
@@ -360,56 +360,21 @@ MagickExport time_t GetMagickTime(void)
       const char
         *source_date_epoch;
 
-      epoch_initialized=MagickTrue;
       source_date_epoch=getenv("SOURCE_DATE_EPOCH");
       if (source_date_epoch != (const char *) NULL)
         {
           time_t
             epoch;
 
-          epoch=(time_t) StringToDouble(source_date_epoch,(char **) NULL);
+          epoch=(time_t) StringToMagickOffsetType(source_date_epoch,100.0);
           if ((epoch > 0) && (epoch <= time((time_t *) NULL)))
-            constant_magick_time=epoch;
+            magick_epoch=epoch;
         }
-    }
-  if (constant_magick_time != 0)
-    return(constant_magick_time);
-  return(time((time_t *) NULL));
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-+   G e t M a g i c k T T L                                                   %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  GetMagickTTL() returns the time as the number of seconds to live.
-%
-%  The format of the GetMagickTTL method is:
-%
-%      MagickOffsetType GetMagickTTL(void)
-%
-*/
-MagickPrivate MagickOffsetType GetMagickTTL(void)
-{
-  static time_t
-    magick_epoch = (time_t) 0;
-
-  static MagickBooleanType
-    epoch_initialized = MagickFalse;
-
-  if (epoch_initialized == MagickFalse)
-    {
       epoch_initialized=MagickTrue;
-      magick_epoch=time((time_t *) NULL);
     }
-  return((MagickOffsetType) GetMagickResourceLimit(TimeResource)-
-    (GetMagickTime()-magick_epoch));
+  if (magick_epoch != 0)
+    return(magick_epoch);
+  return(time((time_t *) NULL));
 }
 
 /*
