@@ -168,6 +168,82 @@ static MagickBooleanType
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+static Image *Read1XImage(Image *image,ExceptionInfo *exception)
+{
+  size_t
+    columns,
+    rows;
+
+  ssize_t
+    i;
+
+  /*
+    Read Windows 1.0 Icon.
+  */
+  (void) ReadBlobLSBLong(image);  /* hot spot X/Y */
+  columns=(size_t) ReadBlobLSBShort(image);
+  rows=(size_t) (ReadBlobLSBShort(image));
+  (void) ReadBlobLSBShort(image);  /* width of bitmap in bytes */
+  (void) ReadBlobLSBShort(image);  /* cursor color */
+  if ((rows != 32 && rows != 64) || (columns != 32 && columns != 64))
+    ThrowImageException(CorruptImageError,"ImproperImageHeader");
+  /*
+    Convert bitmap scanline.
+  */
+  if (SetImageExtent(image,columns,rows,exception) == MagickFalse)
+    return((Image *) NULL);
+  image->alpha_trait=BlendPixelTrait;
+  if (AcquireImageColormap(image,3,exception) == MagickFalse)
+    return((Image *) NULL);
+  image->colormap[1].alpha=TransparentAlpha;
+  for (i=0; i < 2; i++)
+  {
+    ssize_t
+      y;
+
+    for (y=0; y < (ssize_t) image->columns; y++)
+    {
+      Quantum
+        *q;
+
+      ssize_t
+        x;
+
+      q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+      if (q == (Quantum *) NULL)
+        break;
+      for (x=0; x < (ssize_t) (image->columns-7); x+=8)
+      {
+        size_t
+          bit,
+          byte;
+ 
+        byte=(size_t) ReadBlobByte(image);
+        for (bit=0; bit < 8; bit++)
+        {
+          Quantum
+            index;
+
+          index=((byte & (0x80 >> bit)) != 0 ? (i == 0 ? 0x01 : 0x02) : 0x00);
+          if (i == 0)
+            SetPixelIndex(image,index,q);
+          else
+            if (GetPixelIndex(image,q) != 0x01)
+              SetPixelIndex(image,index,q);
+          q+=GetPixelChannels(image);
+        }
+      }
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
+        break;
+    }
+  }
+  (void) SyncImage(image,exception);
+  if (CloseBlob(image) == MagickFalse)
+    return(DestroyImage(image));
+  return(image);
+}
+
 static Image *ReadICONImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -186,16 +262,9 @@ static Image *ReadICONImage(const ImageInfo *image_info,
   MagickSizeType
     extent;
 
-  ssize_t
-    i,
-    x;
-
   Quantum
     *q;
 
-  unsigned char
-    *p;
-
   size_t
     bit,
     byte,
@@ -205,9 +274,14 @@ static Image *ReadICONImage(const ImageInfo *image_info,
 
   ssize_t
     count,
+    i,
     offset,
+    x,
     y;
 
+  unsigned char
+    *p;
+
   /*
     Open image file.
   */
@@ -225,6 +299,9 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     }
   (void) memset(&icon_file,0,sizeof(icon_file));
   icon_file.reserved=(short) ReadBlobLSBShort(image);
+  if ((icon_file.reserved == 0x0001) || (icon_file.reserved == 0x0101) ||
+      (icon_file.reserved == 0x0201))
+    return(Read1XImage(image,exception));
   icon_file.resource_type=(short) ReadBlobLSBShort(image);
   icon_file.count=(short) ReadBlobLSBShort(image);
   if ((icon_file.reserved != 0) ||
@@ -381,8 +458,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
           if (image->colors > GetBlobSize(image))
             ThrowReaderException(CorruptImageError,
               "InsufficientImageDataInFile");
-          if (AcquireImageColormap(image,image->colors,exception) ==
-              MagickFalse)
+          if (AcquireImageColormap(image,image->colors,exception) == MagickFalse)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           icon_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
             image->colors,4UL*sizeof(*icon_colormap));
@@ -812,12 +888,12 @@ static Image *AutoResizeImage(const Image *image,const char *option,
     *images,
     *resized;
 
-  ssize_t
-    i;
-
   size_t
     sizes[MAX_SIZES]={256, 192, 128, 96, 64, 48, 40, 32, 24, 16};
 
+  ssize_t
+    i;
+
   images=NULL;
   *count=0;
   i=0;
@@ -982,7 +1058,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
             return(MagickFalse);
           }
         write_info=CloneImageInfo(image_info);
-        (void) CopyMagickString(write_info->magick,"PNG",MagickPathExtent);
+        (void) CopyMagickString(write_info->magick,"PNG32",MagickPathExtent);
         length=0;
         /*
           Don't write any ancillary chunks except for gAMA.
@@ -991,8 +1067,8 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
         /*
           Only write PNG32 formatted PNG (32-bit RGBA), 8 bits per channel.
         */
-        (void) SetImageArtifact(write_image,"png:format","png32");
-        (void) SetImageArtifact(write_image,"png:color-type","6");
+        (void) SetImageArtifact(write_image,"png:IHDR.color-type-orig","6");
+        (void) SetImageArtifact(write_image,"png:IHDR.bit-depth-orig","8");
         png=(unsigned char *) ImageToBlob(write_info,write_image,&length,
           exception);
         write_image=DestroyImageList(write_image);
